searchNodes=[{"doc":"Caveat emptor: Protean is a library for personal learning and exploration, not (yet) for doing Serious Work . An experimental Elixir library for managing state and side-effects through the use of event-driven statecharts. It is heavily inspired by XState , a robust JavaScript/TypeScript statechart implementation, but strays to adhere to Elixir idioms and OTP conventions. Protean also attempts to follow the SCXML standard, though not completely. What are statecharts? They are an extension to finite state machines that allow you to model complex behavior in a declarative, data-driven manner. They include nested and parallel states, enhanced/augmented state (through context), side-effects (through actions), process management (through invoke), and more. To learn more about statecharts, I recommend statecharts.dev . Goals This project is currently an exploration of statecharts as they fit into the context of Elixir and OTP. XState adopted the actor model in its implementation, so Elixir seemed like a natural fit. However, it may be that Elixir/OTP makes these abstractions unnecessary. Example Add Protean.Supervisor under your application supervisor. This starts a supervisor that is used by Protean internally to manage subprocesses. children = [ Protean.Supervisor , # ... ] This simple statechart has a single state that defines the behavior of a counter with an optional maximum and minimum. defmodule Counter do use Protean alias Protean.Action @machine [ initial : &quot;active&quot; , context : [ count : 0 , min : nil , max : nil ] , states : [ active : [ on : [ { &quot;Inc&quot; , actions : :increment , guard : [ not : :at_max ] } , { &quot;Dec&quot; , actions : :decrement , guard : [ not : :at_min ] } , { match ( { &quot;Set&quot; , _ } ) , actions : :set_min_or_max } , { match ( { &quot;Log&quot; , _ } ) , actions : :log } ] ] ] ] @impl true def handle_action ( :increment , state , _event ) , do : Action . assign_in ( state , [ :count ] , &amp; &amp;1 + 1 ) def handle_action ( :decrement , state , _event ) , do : Action . assign_in ( state , [ :count ] , &amp; &amp;1 - 1 ) def handle_action ( :set_min_or_max , state , { &quot;Set&quot; , { key , val } } ) do state |&gt; Action . assign ( key , val ) end def handle_action ( :log , state , { &quot;Log&quot; , attribute } ) do %{ context : context } = state IO . puts ( &quot; \#{ attribute } : \#{ context [ attribute ] } &quot; ) state end @impl true def guard ( :at_max , %{ context : %{ max : max , count : count } } , _event ) do max &amp;&amp; count &gt;= max end def guard ( :at_min , %{ context : %{ min : min , count : count } } , _event ) do min &amp;&amp; count &lt;= min end end It can be started under a supervisor, but we'll start it directly. { :ok , pid } = Protean . start_link ( Counter ) Protean . current ( pid ) . context # %{count: 0, min: nil, max: nil} Protean . send ( pid , &quot;Inc&quot; ) # :ok Enum . each ( 1 .. 4 , fn _ -&gt; Protean . send ( pid , &quot;Inc&quot; ) end ) Protean . current ( pid ) . context # %{count: 5, min: nil, max: nil} Protean . call ( pid , { &quot;Set&quot; , { :max , 10 } } ) # %Protean.State{ # context: %{count: 5, max: 10, min: nil}, # event: {&quot;Set&quot;, {:max, 10}}, # value: MapSet.new([[&quot;active&quot;, &quot;#&quot;]]) # } Enum . each ( 1 .. 20 , fn _ -&gt; Protean . send ( pid , &quot;Inc&quot; ) end ) Protean . send ( pid , { &quot;Log&quot; , :count } ) # count: 10 Defining a statechart Protean machines are event-driven statecharts , which means that, unlike ordinary finite-state machines, they can have complex, nested, potentially parallel states. This is more easily visualized than read, and I highly recommend looking at XState's introduction to state machines and statecharts for that reason. TODO: States TODO: Transitions TODO: Guards and automatic transitions TODO: Actions TODO: Invoked processes Starting supervised machines Just like GenServer , Protean machines will be most often started under a supervision tree. Invoking use Protean will automatically define a child_spec/1 function that allows you to start the process directly under a supervisor. children = [ Counter ] Supervisor . start_link ( children , strategy : :one_for_one ) Protean machines also accept the same options as Protean.start_link/2 . See those docs for more details. For instance, here's how you could start the Counter with a custom name: children = [ # Start the Counter machine { Counter , name : MyCounter } ] Supervisor . start_link ( children , strategy : :one_for_one ) Protean . current ( MyCounter ) # %Protean.State{ # context: %{count: 0, max: nil, min: nil}, # event: &quot;$protean.init&quot;, # value: MapSet.new([[&quot;active&quot;, &quot;#&quot;]]) # } Interacting with Protean machines Under the hood, a Protean machine is a GenServer , and Protean exposes a similar set of functions for interacting with one. You can see the individual docs for the functions in this module for details on their behavior, but here are some highlights. Familiar functions call/3 - Send an event synchronously to a Protean machine and receive the machine state and any replies resulting from transition. send/2 - Send an event asynchronously to a Protean machine. Always returns :ok . send_after/3 - Send an event to a Protean machine after a given delay. Like Process.send_after/4 , returns a timer reference so that the send can be canceled with Process.cancel_timer/2 . Additional functions specific to Protean machines current/1 - Get the current machine state of a running Protean machine. matches?/2 - Query the currently active state(s) of a machine. subscribe/2 (and unsubscribe/2 ) - Subscribes the calling process to receive a message on every state transition. Protean Supervisor Protean uses a DynamicSupervisor to manage internally spawned processes (often spawned through the use of :invoke ). The simplest thing to do is to add Protean.Supervisor in your application supervision tree: def start ( _type , _args ) do children = [ Protean.Supervisor , # ... ] Supervisor . start_link ( children , strategy : :one_for_one ) end This will start the supervisor under the name Protean.Supervisor and no additional configuration will be required. If you would like to start multiple supervisors, or a different type of supervisor (like a fancy PartitionSupervisor ), you can pass the new name as an option when starting a machine. Here's how that might look using the counter example from before. # in your supervision tree children = [ { Protean.Supervisor , name : ProteanSupervisor1 } , { Protean.Supervisor , name : ProteanSupervisor2 } ] # starting the counter Protean . start_link ( Counter , supervisor : ProteanSupervisor1 ) In the above example, any processes that are spawned by the Protean interpreter running Counter will use ProteanSupervisor1 .","ref":"Protean.html","title":"Protean","type":"behaviour"},{"doc":"Makes a synchronous call to the machine, awaiting any transitions that result. Returns a tuple of {state, replies} , where state is the next state of the machine, and replies is a (possibly empty) list of replies returned by action callbacks resulting from the event.","ref":"Protean.html#call/3","title":"Protean.call/3","type":"function"},{"doc":"Synchronously retrieve the current machine state. TODO: Allow optional timeout as with call/3 .","ref":"Protean.html#current/1","title":"Protean.current/1","type":"function"},{"doc":"Optional callback for defining dynamic delays. Example @machine [ # ... states : [ will_transition : [ after : [ delay : &quot;my_delay&quot; , target : &quot;new_state&quot; ] ] , new_state : [ # ... ] ] ] @impl true def delay ( &quot;my_delay&quot; , state , _ ) do state . context [ :configured_delay ] || 1000 end","ref":"Protean.html#c:delay/3","title":"Protean.delay/3","type":"callback"},{"doc":"Optional callback to determine whether a conditional transition should occur. Example @machine [ # ... states : [ editing_user : [ on : [ { { :user_commit , _ } , guard : :valid_user? , actions : [ &quot;broadcast&quot; ] , target : &quot;viewing_user&quot; } , { { :user_commit , _ } , guard : { :not , :valid_user? } , actions : [ &quot;show_invalid_user_error&quot; ] } ] ] ] ] @impl true def guard ( :valid_user? , state , { _ , user } ) do User . changeset ( % User { } , user ) . valid? end","ref":"Protean.html#c:guard/3","title":"Protean.guard/3","type":"callback"},{"doc":"Optional callback for actions specified in response to a transition. Receives the current machine state and event triggering the action as arguments. Returns one of: state - same as {:noreply, state} {:noreply, state} - the machine state with any new actions {:reply, reply, state} - a reply and the machine state with any new actions Example @machine [ # ... on : [ { match ( { :data , _any } ) , target : :data_received , actions : [ :assign_data , :broadcast_data ] } ] ] @impl true def handle_action ( :assign_data , state , { :data , data } ) do state |&gt; Protean.Action . assign ( :last_received , data ) end def handle_action ( :broadcast_data , state , _ ) do %{ notify : pid , last_received : data } = state . context PubSub . broadcast! ( @pubsub , @topic , data ) state = state |&gt; Protean.Action . send ( { :data , data } , to : pid ) { :reply , data , state } end","ref":"Protean.html#c:handle_action/3","title":"Protean.handle_action/3","type":"callback"},{"doc":"Optional callback for invoked processes specified during machine execution. Should return a value or child specification for the type of process being invoked. Example @machine [ # ... states : [ # ... awaiting_task : [ invoke : [ task : &quot;my_task&quot; , done : &quot;completed&quot; ] ] , completed : [ # ... ] ] ] @impl true def invoke ( &quot;my_task&quot; , _state , event_data ) do { __MODULE__ , :run_my_task , [ event_data ] } end","ref":"Protean.html#c:invoke/3","title":"Protean.invoke/3","type":"callback"},{"doc":"Helper macro to allow match expressions on events during machine definition. Example @machine [ # ... on : [ # Match events that are instances of `MyStruct` { match ( % MyStruct { } ) , target : &quot;...&quot; } , # Match anything { match ( _ ) , target : &quot;...&quot; } ] ]","ref":"Protean.html#match/1","title":"Protean.match/1","type":"macro"},{"doc":"Returns true if the machine is currently in the given state. Note that calling matches?/2 on a machine process is a synchronous operation that is equivalent to: machine |&gt; Protean . current ( ) |&gt; Protean . matches? ( descriptor )","ref":"Protean.html#matches?/2","title":"Protean.matches?/2","type":"function"},{"doc":"Sends an asynchronous event to the machine. Shares semantics with GenServer.cast/2 .","ref":"Protean.html#send/2","title":"Protean.send/2","type":"function"},{"doc":"Sends an event to the machine after time in milliseconds has passed. Returns a timer reference that can be canceled with Process.cancel_timer/1 .","ref":"Protean.html#send_after/3","title":"Protean.send_after/3","type":"function"},{"doc":"Start a Protean machine linked to the current process. This is often used to start the machine as part of a supervision tree. See GenServer.start_link/3 for description of return value. The semantics are similar to GenServer.start_link/3 and accepts the same options, with the addition of some specific to Protean. Options :context - context map that will be merged into the default context defined by the machine. :machine - defaults to module - module used for machine definition. :module - defaults to module - callback module used for actions, guards, invoke, etc. See &quot;Callbacks&quot;. :parent - defaults to self() - process id of the parent that will receive events from the machine if a Protean.Action.send(..., to: :parent) action is used or when the machine reaches a state with :type of :final . :supervisor - defaults to Protean.Supervisor - name of the supervisor process that will be used to start processes resulting from running the machine. See &quot;Supervisor&quot;. Any option accepted by GenServer.start_link/3 .","ref":"Protean.html#start_link/2","title":"Protean.start_link/2","type":"function"},{"doc":"TODO","ref":"Protean.html#stop/3","title":"Protean.stop/3","type":"function"},{"doc":"Subscribes the caller to a running machine, returning a reference. Subscribers will receive messages whenever the machine transitions, as well as a :DOWN message when the machine exits. (This can be controlled with the :monitor option.) Messages are sent in the shape of: { :state , ref , { state , replies } } where: ref is a monitor reference returned by the subscription; state is the machine state resulting from the transition; replies is a (possibly empty) list of replies resulting from actions on transition. If the process is already dead when subscribing, a :DOWN message is delivered immediately. Arguments server - machine to subscribe the caller to; subscribe_to - one of :all (default) or :replies , in which case messages will only be sent to the caller if the replies list is non-empty; options : :monitor - whether to receive a :DOWN message on receive exit (defaults to true ).","ref":"Protean.html#subscribe/3","title":"Protean.subscribe/3","type":"function"},{"doc":"Unsubscribes the caller from the machine.","ref":"Protean.html#unsubscribe/2","title":"Protean.unsubscribe/2","type":"function"},{"doc":"Any message sent to a Protean machine.","ref":"Protean.html#t:event/0","title":"Protean.event/0","type":"type"},{"doc":"Option values for Protean machines.","ref":"Protean.html#t:machine_option/0","title":"Protean.machine_option/0","type":"type"},{"doc":"A running Protean machine process.","ref":"Protean.html#t:server/0","title":"Protean.server/0","type":"type"},{"doc":"Option values for start* functions.","ref":"Protean.html#t:start_option/0","title":"Protean.start_option/0","type":"type"},{"doc":"Option values for subscribe/2 .","ref":"Protean.html#t:subscribe_option/0","title":"Protean.subscribe_option/0","type":"type"},{"doc":"","ref":"Protean.html#t:subscribe_to_option/0","title":"Protean.subscribe_to_option/0","type":"type"},{"doc":"Option values for use Protean .","ref":"Protean.html#t:using_option/0","title":"Protean.using_option/0","type":"type"},{"doc":"Protean manages state, processes, and side-effects through actions , data structures describing things that should occur as a result of a transition. Introduction When a Protean machine transitions from one state to the next (or even self-transitions back to the same state), the interpreter collects any actions that should be performed as a result of that transition. Actions are then executed in a specific order: Exit actions - Any :exit actions specified on states that are exiting as a result of the transition. Transition actions - Any :actions specified on the transition that responded to the event. Entry actions - Any :entry actions specified on the states being entered. Actions are a unifying abstraction that serve many purposes: assign and update a machine's context; produce additional actions to be run immediately after; run arbitrary side-effects, like broadcasting a PubSub message; reply to the sender of the event with a value; spawn additional processes whose lifecycle can be tied to machine execution. I recommend reading the sections below to gain a better understanding of how and where actions can be applied, then review the &quot;Callback Actions&quot; and &quot;Inline Actions&quot; provided by this module. Handling an action The most common way to use actions is through the Protean.handle_action/3 callback. This callback is run when an action is specified like this: [ # ... entry : [ :entering_state ] on : [ { match ( % MyEvent { } ) , actions : [ :first_action , :second_action ] } ] , exit : [ :exiting_state ] ] These are then handled in callbacks: @impl true def handle_action ( :first_action , state , % MyEvent { data : data } ) do # ... { :reply , reply , state } end See Protean.handle_action/3 for possible return values and their effect. Action callbacks must always return the machine state, but they can attach actions to that state that will be immediately executed by the interpreter. For instance: def handle_action ( :update_data , state , { :data_updated , changes } ) do state |&gt; Action . assign_in ( [ :data ] , &amp; Map . merge ( &amp;1 , changes ) ) end In this case, assign_in/3 is being used to update some data in the machine :context . But, we could perform additional actions if we wish, such as: def handle_action ( :update_data , state , { :data_updated , changes } ) do %{ topic : topic , other_process : pid , data : data } = state . context new_data = Map . merge ( data , changes ) PubSub . broadcast! ( @pubsub , topic , { :data_updated , changes } ) state |&gt; Action . send ( { :data_commited , new_data } , to : pid ) |&gt; Action . assign ( :data , new_data ) end Note about return values These last two examples have returned the state directly, which is equivalent to returning {:noreply, state} . You can also emit replies that will be available to the sender of the event using {:reply, reply, state} . This can be very useful for creating client APIs around your machines, much like you would do with GenServer . The best practice is to let the interpreter execute as many actions as possible, as opposed to performing explicit side-effects in the callback. In many cases, such as spawning additional processes, this allows Protean to supervise and tie process lifecycle to the machine's lifecycle (or changes in state). Low-level API: the Action behaviour Ultimately, all actions resolve to Protean.Action.t/0 , a simple struct containing a module and an argument. The module is expected to implement the Protean.Action behaviour. Action execution, then, follows a simple logic: if the action being executed is a Protean.Action struct, call the exec_action/2 callback on the module, passing it the associated argument and the interpreter in its current state. Otherwise, wrap it in a struct that delegates to the callback module associated with the machine. This is how Protean.handle_action/3 is called. Many of features features boil down to syntax sugar over actions, including :invoke and delayed transitions using :after .","ref":"Protean.Action.html","title":"Protean.Action","type":"behaviour"},{"doc":"Create an inline action that applies a function to or merges assigns into state context.","ref":"Protean.Action.html#assign/1","title":"Protean.Action.assign/1","type":"function"},{"doc":"Create an inline action that will assign to state context.","ref":"Protean.Action.html#assign/2","title":"Protean.Action.assign/2","type":"function"},{"doc":"Attach an action that assigns to a state's context.","ref":"Protean.Action.html#assign/3","title":"Protean.Action.assign/3","type":"function"},{"doc":"Create an inline action that will assign into state context. Similar to put_in/3 .","ref":"Protean.Action.html#assign_in/2","title":"Protean.Action.assign_in/2","type":"function"},{"doc":"Attach an action that assigns into a state's context. Similar to put_in/3 .","ref":"Protean.Action.html#assign_in/3","title":"Protean.Action.assign_in/3","type":"function"},{"doc":"Create an inline action that will execute the first of a list of actions whose guard is truthy.","ref":"Protean.Action.html#choose/1","title":"Protean.Action.choose/1","type":"function"},{"doc":"Attach an action that executes the first of a list of actions whose guard is truthy.","ref":"Protean.Action.html#choose/2","title":"Protean.Action.choose/2","type":"function"},{"doc":"Accepts the action_arg passed with the action as well as the Protean interpreter. Returns one of three values: {:cont, interpreter} to continue running the action pipeline {:cont, interpreter, [action]} to inject actions that will be run immediately before running the rest of the pipeline's actions {:halt, interpreter} to halt the action pipeline, canceling any further actions","ref":"Protean.Action.html#c:exec_action/2","title":"Protean.Action.exec_action/2","type":"callback"},{"doc":"Attach a custom action that implements the Protean.Action behaviour.","ref":"Protean.Action.html#put/2","title":"Protean.Action.put/2","type":"function"},{"doc":"Create an inline action that will send a message to a process.","ref":"Protean.Action.html#send/2","title":"Protean.Action.send/2","type":"function"},{"doc":"Attach an action that will send a message to a process.","ref":"Protean.Action.html#send/3","title":"Protean.Action.send/3","type":"function"},{"doc":"Allowed return value from exec_action/2 .","ref":"Protean.Action.html#t:exec_action_return/0","title":"Protean.Action.exec_action_return/0","type":"type"},{"doc":"Data structure representing an executable action.","ref":"Protean.Action.html#t:t/0","title":"Protean.Action.t/0","type":"type"},{"doc":"Protocol for guarded transitions, actions, etc. Default implementations are provided for: BitString - Call callback module with string, state, and event Atom - Call callback module with atom, state, and event Function - Call function with state and event Tuple - Higher-order guard utilities: {:and, [guard1, ...]} {:or, [guard1, ...]} {:not, guard} {:in, query} - Delegates to Protean.State.matches?/2","ref":"Protean.Guard.html","title":"Protean.Guard","type":"protocol"},{"doc":"","ref":"Protean.Guard.html#allows?/4","title":"Protean.Guard.allows?/4","type":"function"},{"doc":"Implements Protean.Guard protocol.","ref":"Protean.Guard.html#t:guard/0","title":"Protean.Guard.guard/0","type":"type"},{"doc":"","ref":"Protean.Guard.html#t:t/0","title":"Protean.Guard.t/0","type":"type"},{"doc":"","ref":"Protean.MachineConfig.html","title":"Protean.MachineConfig","type":"module"},{"doc":"Data structure representing a Protean machine.","ref":"Protean.MachineConfig.html#__struct__/0","title":"Protean.MachineConfig.__struct__/0","type":"function"},{"doc":"Compute the full set of active nodes for the given states.","ref":"Protean.MachineConfig.html#active/2","title":"Protean.MachineConfig.active/2","type":"function"},{"doc":"Fetch a node by its id. Raises if the node cannot be found.","ref":"Protean.MachineConfig.html#fetch!/2","title":"Protean.MachineConfig.fetch!/2","type":"function"},{"doc":"Compute the initial state for a machine configuration, including any entry actions that result from entering the default states.","ref":"Protean.MachineConfig.html#initial_state/1","title":"Protean.MachineConfig.initial_state/1","type":"function"},{"doc":"Return the full lineage of the given id, including itself and all of its ancestors.","ref":"Protean.MachineConfig.html#lineage/2","title":"Protean.MachineConfig.lineage/2","type":"function"},{"doc":"","ref":"Protean.MachineConfig.html#new/2","title":"Protean.MachineConfig.new/2","type":"function"},{"doc":"User-defined machine configuration.","ref":"Protean.MachineConfig.html#t:config/0","title":"Protean.MachineConfig.config/0","type":"type"},{"doc":"Internal representation of a Protean machine","ref":"Protean.MachineConfig.html#t:t/0","title":"Protean.MachineConfig.t/0","type":"type"},{"doc":"Snapshot of active states, context, and the latest event seen by the machine. This module is rarely interacted with directly. Instead, use the API exposed by the top-level Protean , such as Protean.matches?/2 , or actions exposed by Protean.Action .","ref":"Protean.State.html","title":"Protean.State","type":"module"},{"doc":"","ref":"Protean.State.html#assign/2","title":"Protean.State.assign/2","type":"function"},{"doc":"","ref":"Protean.State.html#matches?/2","title":"Protean.State.matches?/2","type":"function"},{"doc":"","ref":"Protean.State.html#t:context/0","title":"Protean.State.context/0","type":"type"},{"doc":"","ref":"Protean.State.html#t:private_state/0","title":"Protean.State.private_state/0","type":"opaque"},{"doc":"","ref":"Protean.State.html#t:t/0","title":"Protean.State.t/0","type":"type"},{"doc":"","ref":"Protean.State.html#t:value/0","title":"Protean.State.value/0","type":"type"},{"doc":"TODO Event descriptors Guards Internal Exact Actions","ref":"Protean.Transition.html","title":"Protean.Transition","type":"module"},{"doc":"Return the actions associated with a transition","ref":"Protean.Transition.html#actions/1","title":"Protean.Transition.actions/1","type":"function"},{"doc":"","ref":"Protean.Transition.html#domain/1","title":"Protean.Transition.domain/1","type":"function"},{"doc":"Checks whether the transition is enabled for the given event.","ref":"Protean.Transition.html#enabled?/4","title":"Protean.Transition.enabled?/4","type":"function"},{"doc":"","ref":"Protean.Transition.html#new/1","title":"Protean.Transition.new/1","type":"function"},{"doc":"","ref":"Protean.Transition.html#t:t/0","title":"Protean.Transition.t/0","type":"type"},{"doc":"Execution logic for a Protean machine.","ref":"Protean.Interpreter.html","title":"Protean.Interpreter","type":"module"},{"doc":"Handle an event, executing any transitions, actions, and side-effects associated with the current machine state. Returns a tuple of the interpreter and any replies resulting from actions that were run.","ref":"Protean.Interpreter.html#handle_event/2","title":"Protean.Interpreter.handle_event/2","type":"function"},{"doc":"Create a new Interpreter . The returned interpreter will still need to be started, which could result in additional side-effects. See start/1 .","ref":"Protean.Interpreter.html#new/1","title":"Protean.Interpreter.new/1","type":"function"},{"doc":"Whether the interpreter has been started and can accept events.","ref":"Protean.Interpreter.html#running?/1","title":"Protean.Interpreter.running?/1","type":"function"},{"doc":"Entrypoint for the interpreter that must be called before the interpreter will be in a state where it can handle external events. This is necessary in order to handle any initializing actions, invokes, or automatic transitions. Calling start/1 on an already-running interpreter is a no-op.","ref":"Protean.Interpreter.html#start/1","title":"Protean.Interpreter.start/1","type":"function"},{"doc":"Return the current machine state.","ref":"Protean.Interpreter.html#state/1","title":"Protean.Interpreter.state/1","type":"function"},{"doc":"Stop an interpreter, preventing further event processing and terminating any invoked processes.","ref":"Protean.Interpreter.html#stop/1","title":"Protean.Interpreter.stop/1","type":"function"},{"doc":"","ref":"Protean.Interpreter.html#subscribe/2","title":"Protean.Interpreter.subscribe/2","type":"function"},{"doc":"","ref":"Protean.Interpreter.html#unsubscribe/2","title":"Protean.Interpreter.unsubscribe/2","type":"function"},{"doc":"Sets the context of the current state.","ref":"Protean.Interpreter.html#with_context/2","title":"Protean.Interpreter.with_context/2","type":"function"},{"doc":"","ref":"Protean.Interpreter.html#t:invoked/0","title":"Protean.Interpreter.invoked/0","type":"type"},{"doc":"","ref":"Protean.Interpreter.html#t:invoked_id/0","title":"Protean.Interpreter.invoked_id/0","type":"type"},{"doc":"","ref":"Protean.Interpreter.html#t:invoked_service/0","title":"Protean.Interpreter.invoked_service/0","type":"opaque"},{"doc":"","ref":"Protean.Interpreter.html#t:t/0","title":"Protean.Interpreter.t/0","type":"type"},{"doc":"Purely-functional statechart core based on the SCXML specification. Provides the underlying state-transition logic for a statechart, primarily through transition/3 , a higher-level transition API for using %Protean.Machine{} apart from the interpreter provided by Protean, and take_transitions/3 , a lower-level API used by a statechart interpreter. It is uncommon to use this module independently of the Protean behaviour.","ref":"Protean.Machinery.html","title":"Protean.Machinery","type":"module"},{"doc":"Select any machine transitions that apply to the given event in the current state.","ref":"Protean.Machinery.html#select_transitions/4","title":"Protean.Machinery.select_transitions/4","type":"function"},{"doc":"","ref":"Protean.Machinery.html#take_transitions/3","title":"Protean.Machinery.take_transitions/3","type":"function"},{"doc":"Transition in response to an event.","ref":"Protean.Machinery.html#transition/3","title":"Protean.Machinery.transition/3","type":"function"},{"doc":"Internal representation of an individual node in a Protean.MachineConfig struct.","ref":"Protean.Node.html","title":"Protean.Node","type":"module"},{"doc":"Given a Node id, return a list containing that id and all of its ancestors.","ref":"Protean.Node.html#ancestor_ids/1","title":"Protean.Node.ancestor_ids/1","type":"function"},{"doc":"","ref":"Protean.Node.html#common_ancestor_id/1","title":"Protean.Node.common_ancestor_id/1","type":"function"},{"doc":"","ref":"Protean.Node.html#common_ancestor_id/2","title":"Protean.Node.common_ancestor_id/2","type":"function"},{"doc":"","ref":"Protean.Node.html#complex?/1","title":"Protean.Node.complex?/1","type":"function"},{"doc":"Tests whether descendant_id is in fact a descendant of ancestor_id . Returns false if they are the same node.","ref":"Protean.Node.html#descendant?/2","title":"Protean.Node.descendant?/2","type":"function"},{"doc":"Return the entry actions associated with the node.","ref":"Protean.Node.html#entry_actions/1","title":"Protean.Node.entry_actions/1","type":"function"},{"doc":"Sort the given nodes in entry order.","ref":"Protean.Node.html#entry_order/1","title":"Protean.Node.entry_order/1","type":"function"},{"doc":"Return the exit actions associated with the node.","ref":"Protean.Node.html#exit_actions/1","title":"Protean.Node.exit_actions/1","type":"function"},{"doc":"Sort the given nodes in exit order.","ref":"Protean.Node.html#exit_order/1","title":"Protean.Node.exit_order/1","type":"function"},{"doc":"","ref":"Protean.Node.html#leaf?/1","title":"Protean.Node.leaf?/1","type":"function"},{"doc":"Given a Node id, return its parent's id. Returns nil if the Node is the root.","ref":"Protean.Node.html#parent_id/1","title":"Protean.Node.parent_id/1","type":"function"},{"doc":"Resolve a Node to its leaves (atomic or final) by either returning the given node or following the node's children.","ref":"Protean.Node.html#resolve_to_leaves/1","title":"Protean.Node.resolve_to_leaves/1","type":"function"},{"doc":"An atomic node is a node without child states.","ref":"Protean.Node.html#t:atomic/0","title":"Protean.Node.atomic/0","type":"type"},{"doc":"A complex Node is one that has child nodes.","ref":"Protean.Node.html#t:complex/0","title":"Protean.Node.complex/0","type":"type"},{"doc":"A compound node is a node that defines children, of which only one can be active. It must additionally define an :initial attribute, the id of the child state that should default to active if the compound state is entered.","ref":"Protean.Node.html#t:compound/0","title":"Protean.Node.compound/0","type":"type"},{"doc":"A final node is a type of atomic node that represents some form of completion, and can therefore define no transitions itself. Note, however, that activating a final node causes an event to be dispatched that a parent node can choose to handle.","ref":"Protean.Node.html#t:final/0","title":"Protean.Node.final/0","type":"type"},{"doc":"ID encompasses the node and all its ancestors. For example, a node :child_a defined as a child of a node :parent_a might have the id [:child_a, :parent_a]","ref":"Protean.Node.html#t:id/0","title":"Protean.Node.id/0","type":"opaque"},{"doc":"A leaf Node is one that cannot have child states.","ref":"Protean.Node.html#t:leaf/0","title":"Protean.Node.leaf/0","type":"type"},{"doc":"","ref":"Protean.Node.html#t:nodes/0","title":"Protean.Node.nodes/0","type":"type"},{"doc":"A parallel node defines children, all of which are entered when the parallel node is entered.","ref":"Protean.Node.html#t:parallel/0","title":"Protean.Node.parallel/0","type":"type"},{"doc":"A simple Node is one that cannot have child nodes.","ref":"Protean.Node.html#t:simple/0","title":"Protean.Node.simple/0","type":"type"},{"doc":"A Node is a node in a nested state machine. See the type docs for individual nodes for more details.","ref":"Protean.Node.html#t:t/0","title":"Protean.Node.t/0","type":"type"},{"doc":"Supervisor used by the Protean interpreter to manage subprocesses. See the &quot;Supervisors&quot; section of the Protean documentation for more details.","ref":"Protean.Supervisor.html","title":"Protean.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Protean.Supervisor.html#child_spec/1","title":"Protean.Supervisor.child_spec/1","type":"function"},{"doc":"","ref":"Protean.Supervisor.html#start_child/2","title":"Protean.Supervisor.start_child/2","type":"function"},{"doc":"","ref":"Protean.Supervisor.html#start_link/1","title":"Protean.Supervisor.start_link/1","type":"function"},{"doc":"","ref":"Protean.Supervisor.html#terminate_child/2","title":"Protean.Supervisor.terminate_child/2","type":"function"},{"doc":"","ref":"Protean.ConfigError.html","title":"Protean.ConfigError","type":"exception"}]